/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvMath.h"
#include "HvSignalLine.h"
#include "HvSignalVar.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_gkLlBxJu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YhExGCuj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_UsqcPOX1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_08oyGrgo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_J4kA70ne_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mIk04Lla_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zK9kl0qs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_C6HtF3jL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6E3JCOXR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jaRLRNQU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QfMlP5Fh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_L7a2fHtb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_kDeEBer5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_heIsQGD5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_NMOWmtRO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_V8VYodw3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_XxXi7je2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DvID6I2Q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_L41TTmdS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_o5NhZnx4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_xjADUTzN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jnQETobw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_TWUuBt41_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_M3fLaJ3p_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_MlCKPZui_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_a7iQgI1U_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_doGQOgHP;
  SignalLine sLine_k3VbU97H;
  SignalLine sLine_U6aslTJ7;
  ControlBinop cBinop_gkLlBxJu;
  ControlBinop cBinop_YhExGCuj;
  ControlBinop cBinop_UsqcPOX1;
  ControlBinop cBinop_08oyGrgo;
  ControlBinop cBinop_zK9kl0qs;
  ControlBinop cBinop_6E3JCOXR;
  ControlBinop cBinop_jaRLRNQU;
  ControlBinop cBinop_QfMlP5Fh;
  ControlVar cVar_L7a2fHtb;
  ControlVar cVar_kDeEBer5;
  SignalVarf sVarf_n1Qbodif;
  SignalVarf sVarf_I2ETkBUj;
  SignalVarf sVarf_otZ2ogru;
  SignalVarf sVarf_r68NMgME;
  ControlBinop cBinop_DvID6I2Q;
  ControlBinop cBinop_jnQETobw;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
